/**
*
*/
import RTIcap.*;
import npacrti.*;

import java.util.*;

/*
*/
public class 
ObjectManager extends _ObjectManagerInterfaceImplBase
              implements RTIDebug
{

    ObjectClassNaming objNaming;
    int MAX_OBJECTS_PER_FEDERATE = 1000;
    int numberOfObjects = 0;
    int eventCount = 0;
    Hashtable objIdToClassId = new Hashtable();

    /**
    */
    public 
    ObjectManager(ObjectClassNaming _objNaming)
    {
        objNaming = _objNaming;
    }// end of CONSTRUCTOR

    /**
    * 
    */
    public
    void requestID(short idCount, org.omg.CORBA.IntHolder firstID, 
                   org.omg.CORBA.IntHolder lastID)
         throws RTIcap.TooManyIDsRequested, RTIcap.IDsupplyExhausted, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       numberOfObjects++;
       firstID.value = numberOfObjects;
       numberOfObjects += idCount;
       lastID.value = numberOfObjects;
       if(DEBUG)
       {
          System.out.println(" firstID "+firstID.value);
          System.out.println(" lastID "+lastID.value);
       }

    }// end of requestID()

    /**
    * 6.2
    */
    public
    void registerObject(short theClass, int theObject,
                FederateAmbassador _fedHandle, short _fedId )
         throws RTIcap.ObjectClassNotDefined, RTIcap.ObjectClassNotPublished, 
                RTIcap.InvalidObjectID, RTIcap.ObjectAlreadyRegistered, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       if(DEBUG) System.out.println(" in registerObject() for class "+theClass+" Object "+theObject);
       ObjectRecord rec = objNaming.getObjectRecord(theClass); 

       rec.register_object(theObject,_fedHandle,_fedId);

       objIdToClassId.put(new Integer(theObject),new Short(theClass));
       if(DEBUG) System.out.println(" out of registerObject() ");

    }// end of registerObject()

    /**
    * 6.4
    */
    public
    RTIcap.EventRetractionHandle updateAttributeValues(int theObject, 
                RTIcap.AttributeHandleValuePair[] theAttributes, 
                double theTime, String theTag,
                FederateAmbassador _fedHandle, short _fedId )
         throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
                RTIcap.AttributeNotOwned, RTIcap.InvalidFederationTime, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       if(DEBUG) System.out.println(" in updateAttributeValues() of ObjectManager for "+theObject);

       RTIcap.EventRetractionHandle ret = null;
       Short theClassId = (Short) objIdToClassId.get(new Integer(theObject));

       if(theClassId!=null)
       {
           eventCount++;
           ret = new EventRetractionHandle(theTime,eventCount,_fedId);

           ObjectRecord rec = objNaming.getObjectRecord(theClassId.shortValue());

           if(rec==null) throw new RTIcap.ObjectNotKnown();

           rec.updateAttributeValues(theObject,theAttributes,theTime,theTag,_fedHandle,_fedId);
       } else throw new RTIcap.ObjectNotKnown();

       if(DEBUG) System.out.println(" out of updateAttributeValues() of ObjectManager ");
       return ret;
    }

    /**
    * 6.6
    */
    public RTIcap.EventRetractionHandle 
    sendInteraction(short theInteraction, 
                    RTIcap.ParameterHandleValuePair[] theParameters, 
                    double theTime, String theTag)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.InteractionClassNotPublished, 
                RTIcap.InteractionParameterNotDefined, 
                RTIcap.InvalidFederationTime, RTIcap.FederateNotExecutionMember,
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       return null;
    }

    /**
    * 6.8
    */
    public RTIcap.EventRetractionHandle 
    deleteObject( int objectID, double theTime, String theTag, 
                  FederateAmbassador fedAmb, short _fedHandle)
         throws RTIcap.ObjectNotKnown, RTIcap.DeletePrivilegeNotHeld, 
                RTIcap.InvalidFederationTime, RTIcap.FederateNotExecutionMember,  
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
       RTIcap.EventRetractionHandle ret = null;
       eventCount++;
       ret = new EventRetractionHandle(theTime,eventCount,_fedHandle);
       //objNaming.delete();

       return ret;

    }// end of deleteObject()

    /**
    * 6.11
    */
    public
    void changeAttributeTransportType(int theObject, short[] theAttributes, 
                                      RTIcap.TransportType theType)
         throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
                RTIcap.AttributeNotOwned, RTIcap.InvalidTransportType, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    */
    public
    void changeAttributeOrderType(int theObject, short[] theAttributes, 
                                  RTIcap.OrderType theType)
         throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotDefined, 
                RTIcap.AttributeNotOwned, RTIcap.InvalidOrderType, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    * 6.12
    */
    public
    void changeInteractionTransportType(short theClass, RTIcap.TransportType theType)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.InteractionClassNotPublished, 
                RTIcap.InvalidTransportType, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    */
    public
    void changeInteractionOrderType(short theClass, RTIcap.OrderType theType)
         throws RTIcap.InteractionClassNotDefined, 
                RTIcap.InteractionClassNotPublished, RTIcap.InvalidOrderType, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    * 6.15
    */
    public
    void requestObjectAttributeValueUpdate1(int theObject, short[] theAttributes)
         throws RTIcap.AttributeNotDefined, RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    * 6.15
    */
    public
    void requestObjectAttributeValueUpdate2(int theObject)
         throws RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    */
    public
    void requestClassAttributeValueUpdate1(short theClass, short[] theAttributes)
         throws RTIcap.ObjectClassNotDefined, RTIcap.AttributeNotDefined, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {

    }

    /**
    */
    public
    void requestClassAttributeValueUpdate2(short theClass)
         throws RTIcap.ObjectClassNotDefined, RTIcap.FederateNotExecutionMember,
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

    /**
    */
    public
    void retract(RTIcap.EventRetractionHandle theHandle)
         throws RTIcap.InvalidRetractionHandle, 
                RTIcap.FederateNotExecutionMember, 
                RTIcap.ConcurrentAccessAttempted, RTIcap.RTIinternalError, 
                RTIcap.Deadlock
    {
    }

}// end of ObjectManager

